---
meta:
    description:
        A DynamoDB table, maybe with DAX, maybe with API Gateway proxy

    parameters:
        client_keys:
            description: a list of client keys with specific usage plans
            value: []

        api_monthly_quota_limit:
            description: the cap on monthly API requests
            value: 

        api_dayly_quota_limit:
            description: the cap on daily API requests
            value:

        api_burst_limit:
            description: The maximum API request rate limit over a time ranging from one to a few seconds.
            value:

        api_rate_limit:
            description: The API request steady-state rate limit (average requests per second over an extended period of time.
            value:

        table_name:
            description: the name of the backend DynamoDB table
            value: {{__context.environment.name + '-' + __context.layer.name + '-' + __context.stage}}

        default_resource:
            description: the root resource will become an alias of this resource
            value:

        resources:
            description: a list of API resources, each associated to a separate DynamoDB table
            value:
                - default

        has_public_api:
            description: set to yes if you want to deploy an API GTW REST API
            value: no

        has_api_cache:
            description: whether the API has a cache
            value: no

        api_cache_size:
            description: the size in Gb of the cache cluster associated to the API
            value: 0.5

        api_cache_ttl:
            description: TTL for the API cache, in seconds
            value: 3600

        key_attribute:
            description: the name of the attribute used as a DynamoDB key
            value: key

        value_attribute:
            description: the name of the attribute to return as response
            value: value

        namespace_parameter:
            description: list of request parameters that define a key namespace
            value:

        api_description:
            description: A description to associate to the created API
            value:

        api_key_required:
            description: Whether an API is required to access the API
            value: yes

        request_parameters:
            description: A map of request parameters
            value: {}

        hosted_zone_id:
            description: The hosted zone ID where the domain name will be recorded
            value:

        domain_name:
            description: Domain name to associate to the root resource
            value:

        certificate_arn:
            description: ARN of the Cert for the domain name.
            value:

        stream_name:
            description: The name of the target Kinesis stream
            value:

        partition_key_path:
            description:
                A global value for partition_key_path. Overriden at the path level.
            value:
                # Works well with Polku's SDKs
                anonymousId

        partition_key:
            description:
                A global value for the partition key. Overriden at the path level.
            value:

        sent_at_path:
            description:
                A global value for sent_at_ts_path. Overriden at the path level.
            value:
                sentAt

        read_capacity:
            description: The DynamoDB read capacity
            value: 1

        write_capacity:
            description: The DynamoDB write capacity
            value: 1

        dax_subnets:
            description: List of subnet IDs onto which a DAX cluster will be deployed
            value: []

        dax_node_type:
            description: The type of the DAX nodes
            value: "dax.r3.large"

        dax_replication_factor:
            description: The replication factor for the DAX cluster
            value: 1
